searchNodes=[{"doc":"ExUnit assertions. See also: siiibo/assert_match which is similar to this module's assert_eq function but with pattern matching.","ref":"Moar.Assertions.html","title":"Moar.Assertions","type":"module"},{"doc":"Asserts that the left and right values are equal. Returns the left value unless the assertion fails, or unless the :returning option is used. Uses assert left == right under the hood. Style note: the authors prefer to use assert in most cases, using assert_eq only when the extra options are helpful or when they want to make assertions in a pipeline. Options: except: ~w[a b]a - when comparing maps, remove these keys from each map if present. ignore_order: boolean - if the left and right values are lists, ignores the order when checking equality. ignore_whitespace: :leading_and_trailing - if the left and right values are strings, ignores leading and trailing space when checking equality. only: ~w[a b]a - when comparing maps, filter each map to only these keys. returning: value - returns value if the assertion passes, rather than returning the left value. within: delta - asserts that the left and right values are within delta of each other. within: {delta, time_unit} - like within: delta but performs time comparisons in the specified time_unit . See Moar.Duration for more about time units. If left and right are strings, they are parsed as ISO8601 dates. Examples iex&gt; import Moar.Assertions iex&gt; %{ a : 1 } |&gt; Map . put ( :b , 2 ) |&gt; assert_eq ( %{ a : 1 , b : 2 } ) %{ a : 1 , b : 2 } iex&gt; assert_eq ( [ 1 , 2 ] , [ 2 , 1 ] , ignore_order : true ) [ 1 , 2 ] iex&gt; assert_eq ( &quot;foo bar&quot; , &quot; foo bar \\n &quot; , ignore_whitespace : :leading_and_trailing ) &quot;foo bar&quot; iex&gt; map = %{ a : 1 , b : 2 } iex&gt; map |&gt; Map . get ( :a ) |&gt; assert_eq ( 1 , returning : map ) %{ a : 1 , b : 2 } iex&gt; assert_eq ( 4 / 28 , 0.14 , within : 0.01 ) 0.14285714285714285 iex&gt; inserted_at = ~U[2022-01-02 03:00:00Z] iex&gt; updated_at = ~U[2022-01-02 03:04:00Z] iex&gt; assert_eq ( inserted_at , updated_at , within : { 10 , :minute } ) ~U[2022-01-02 03:00:00Z] iex&gt; inserted_at = &quot;2022-01-02T03:00:00Z&quot; iex&gt; updated_at = &quot;2022-01-02T03:04:00Z&quot; iex&gt; assert_eq ( inserted_at , updated_at , within : { 10 , :minute } ) &quot;2022-01-02T03:00:00Z&quot;","ref":"Moar.Assertions.html#assert_eq/3","title":"Moar.Assertions.assert_eq/3","type":"function"},{"doc":"Asserts that datetime is within recency of now (in UTC), returning datetime if the assertion succeeeds. Uses assert_eq(datetime, now, within: recency) under the hood. datetime can be a DateTime , a NaiveDateTime , or an ISO8601-formatted UTC datetime string. recency is a Moar.Duration and defaults to {10, :second} . iex&gt; five_seconds_ago = Moar.DateTime . add ( DateTime . utc_now ( ) , { - 5 , :second } ) iex&gt; assert_recent five_seconds_ago iex&gt; twenty_seconds_ago = Moar.DateTime . add ( DateTime . utc_now ( ) , { - 20 , :second } ) iex&gt; assert_recent twenty_seconds_ago , { 25 , :second }","ref":"Moar.Assertions.html#assert_recent/2","title":"Moar.Assertions.assert_recent/2","type":"function"},{"doc":"Asserts that a pre-condition and a post-condition are true after performing an action, returning the result of the action. To use an anonymous function as the action, wrap it in parentheses and call it with .() . Examples iex&gt; { :ok , agent } = Agent . start ( fn -&gt; 0 end ) ...&gt; iex&gt; assert_that Agent . update ( agent , fn s -&gt; s + 1 end ) , ...&gt; changes : Agent . get ( agent , fn s -&gt; s end ) , ...&gt; from : 0 , ...&gt; to : 1 :ok ...&gt; iex&gt; assert_that Agent . update ( agent , fn s -&gt; s + 1 end ) , ...&gt; changes : Agent . get ( agent , fn s -&gt; s end ) , ...&gt; to : 2 :ok ...&gt; iex&gt; assert_that ( fn -&gt; Agent . update ( agent , fn s -&gt; s + 1 end ) end ) . ( ) , ...&gt; changes : Agent . get ( agent , fn s -&gt; s end ) , ...&gt; to : 3 :ok","ref":"Moar.Assertions.html#assert_that/2","title":"Moar.Assertions.assert_that/2","type":"macro"},{"doc":"Refute that a condition is changed after performing an action, returning the result of the action. Examples iex&gt; { :ok , agent } = Agent . start ( fn -&gt; 0 end ) ...&gt; iex&gt; refute_that Function . identity ( 1 ) , ...&gt; changes : Agent . get ( agent , fn s -&gt; s end ) 1 iex&gt; refute_that Function . identity ( 5 ) , ...&gt; changes : %{ a : 1 } 5","ref":"Moar.Assertions.html#refute_that/2","title":"Moar.Assertions.refute_that/2","type":"macro"},{"doc":"","ref":"Moar.Assertions.html#t:assert_eq_opts/0","title":"Moar.Assertions.assert_eq_opts/0","type":"type"},{"doc":"Atom-related functions.","ref":"Moar.Atom.html","title":"Moar.Atom","type":"module"},{"doc":"Given an atom, returns the atom. Given a string, converts it into an atom, converting non-alphanumeric characters into underscores (via Moar.String.slug/2 ). Examples iex&gt; Moar.Atom . atomize ( :my_atom ) :my_atom iex&gt; Moar.Atom . atomize ( &quot;my-atom&quot; ) :my_atom","ref":"Moar.Atom.html#atomize/1","title":"Moar.Atom.atomize/1","type":"function"},{"doc":"Converts a string to an atom (via String.to_atom/1 ), and returns atoms unchanged. Useful when you aren't sure ahead of time whether you have a string or an atom. Examples iex&gt; Moar.Atom . from_string ( &quot;foo&quot; ) :foo iex&gt; Moar.Atom . from_string ( :bar ) :bar iex&gt; Moar.Atom . from_string ( nil ) ** (ArgumentError) Unable to convert nil into an atom","ref":"Moar.Atom.html#from_string/1","title":"Moar.Atom.from_string/1","type":"function"},{"doc":"Converts a string to an existing atom (via String.to_existing_atom/1 ), and returns atoms unchanged. Useful when you aren't sure ahead of time whether you have a string or an atom. Examples iex&gt; Moar.Atom . to_existing_atom ( &quot;foo&quot; ) :foo iex&gt; assert_raise ArgumentError , ~r/not an already existing atom/ , fn -&gt; ...&gt; Moar.Atom . to_existing_atom ( &quot;sadfasfsfasf&quot; ) ...&gt; end iex&gt; Moar.Atom . to_existing_atom ( :baz ) :baz iex&gt; Moar.Atom . to_existing_atom ( nil ) nil","ref":"Moar.Atom.html#to_existing_atom/1","title":"Moar.Atom.to_existing_atom/1","type":"function"},{"doc":"Converts an atom to a string (via Atom.to_string/1 ), and returns strings unchanged. Useful when you aren't sure ahead of time whether you have a string or an atom. Examples iex&gt; Moar.Atom . to_string ( &quot;foo&quot; ) &quot;foo&quot; iex&gt; Moar.Atom . to_string ( :bar ) &quot;bar&quot; iex&gt; Moar.Atom . to_string ( nil ) ** (ArgumentError) Unable to convert nil into a string","ref":"Moar.Atom.html#to_string/1","title":"Moar.Atom.to_string/1","type":"function"},{"doc":"DateTime-related functions. See also Moar.NaiveDateTime .","ref":"Moar.DateTime.html","title":"Moar.DateTime","type":"module"},{"doc":"Adds duration to date_time . See also subtract/1 and Moar.NaiveDateTime.add/2 . Note This function is naive and intentionally doesn't account for real-world calendars and all of their complexity, such as leap years, leap days, daylight saving time, past and future calendar oddities, etc. As &quot;Falsehoods programmers believe about time&quot; says, &quot;If you think you understand everything about time, you're probably doing it wrong.&quot; See Cldr.Calendar.plus/2 for one example of a function that is far more likely to be correct. iex&gt; start = ~U[2022-01-01T00:00:00Z] iex&gt; Moar.DateTime . add ( start , { 3 , :minute } ) ~U[2022-01-01T00:03:00Z]","ref":"Moar.DateTime.html#add/2","title":"Moar.DateTime.add/2","type":"function"},{"doc":"Returns true if date_time is inclusively inside range which is a tuple containing a start datetime and an end datetime. iex&gt; Moar.DateTime . between? ( ~U[2022-01-01T02:00:00Z] , { ~U[2022-01-01T01:00:00Z] , ~U[2022-01-01T03:00:00Z] } ) true iex&gt; Moar.DateTime . between? ( ~U[2022-01-01T04:00:00Z] , { ~U[2022-01-01T01:00:00Z] , ~U[2022-01-01T03:00:00Z] } ) false","ref":"Moar.DateTime.html#between?/2","title":"Moar.DateTime.between?/2","type":"function"},{"doc":"Like DateTime.from_iso8601/1 but raises if the string cannot be parsed. iex&gt; Moar.DateTime . from_iso8601! ( &quot;2022-01-01T00:00:00Z&quot; ) ~U[2022-01-01T00:00:00Z] iex&gt; Moar.DateTime . from_iso8601! ( &quot;2022-01-01T00:00:00+0800&quot; ) ** (ArgumentError) Expected &quot;2022-01-01T00:00:00+0800&quot; to have a UTC offset of 0, but was: 28800 iex&gt; Moar.DateTime . from_iso8601! ( &quot;Next Thursday after lunch&quot; ) ** (ArgumentError) Invalid ISO8601 format: &quot;Next Thursday after lunch&quot;","ref":"Moar.DateTime.html#from_iso8601!/1","title":"Moar.DateTime.from_iso8601!/1","type":"function"},{"doc":"Returns true if date_time is no older than duration ago (which defaults to 1 minute). iex&gt; Moar.DateTime . recent? ( Moar.DateTime . utc_now ( minus : { 30 , :second } ) ) true iex&gt; Moar.DateTime . recent? ( Moar.DateTime . utc_now ( minus : { 5 , :minute } ) ) false iex&gt; Moar.DateTime . recent? ( Moar.DateTime . utc_now ( minus : { 5 , :minute } ) , { 1 , :hour } ) true","ref":"Moar.DateTime.html#recent?/2","title":"Moar.DateTime.recent?/2","type":"function"},{"doc":"Subtracts duration from date_time . See also add/1 and Moar.NaiveDateTime.subtract/2 . Note This function is naive and intentionally doesn't account for real-world calendars and all of their complexity, such as leap years, leap days, daylight saving time, past and future calendar oddities, etc. As &quot;Falsehoods programmers believe about time&quot; says, &quot;If you think you understand everything about time, you're probably doing it wrong.&quot; See Cldr.Calendar.minus/4 for one example of a function that is far more likely to be correct. iex&gt; start = ~U[2022-01-01T00:03:00Z] iex&gt; Moar.DateTime . subtract ( start , { 3 , :minute } ) ~U[2022-01-01T00:00:00Z]","ref":"Moar.DateTime.html#subtract/2","title":"Moar.DateTime.subtract/2","type":"function"},{"doc":"Like DateTime.to_iso8601/1 but rounds to the nearest second first. iex&gt; Moar.DateTime . to_iso8601_rounded ( ~U[2022-01-01T01:02:03.456789Z] ) &quot;2022-01-01T01:02:03Z&quot;","ref":"Moar.DateTime.html#to_iso8601_rounded/1","title":"Moar.DateTime.to_iso8601_rounded/1","type":"function"},{"doc":"Returns the current UTC time plus or minus the given duration. iex&gt; Moar.DateTime . utc_now ( plus : { 10500 , :millisecond } ) ...&gt; |&gt; Moar.Duration . format ( [ :approx , :from_now ] ) &quot;10 seconds from now&quot; iex&gt; Moar.DateTime . utc_now ( minus : { 10 , :second } ) ...&gt; |&gt; Moar.Duration . format ( [ :approx , :ago ] ) &quot;10 seconds ago&quot;","ref":"Moar.DateTime.html#utc_now/1","title":"Moar.DateTime.utc_now/1","type":"function"},{"doc":"A protocol that defines diff/2 for finding the difference between two terms. Includes implementations for DateTime , NaiveDateTime , and BitString .","ref":"Moar.Difference.html","title":"Moar.Difference","type":"protocol"},{"doc":"Returns the difference between a and b . The fallback implementation uses Kernel.-/2 to subtract b from a .","ref":"Moar.Difference.html#diff/2","title":"Moar.Difference.diff/2","type":"function"},{"doc":"","ref":"Moar.Difference.html#t:t/0","title":"Moar.Difference.t/0","type":"type"},{"doc":"A duration is a {time, unit} tuple. The time is a number and the unit is one of: :nanosecond :microsecond :millisecond :second :minute :hour :day :approx_month (30 days) :approx_year (360 days) Note This module is naive and intentionally doesn't account for real-world calendars and all of their complexity, such as leap years, leap days, daylight saving time, past and future calendar oddities, etc. As &quot;Falsehoods programmers believe about time&quot; says, &quot;If you think you understand everything about time, you're probably doing it wrong.&quot; See Cldr.Calendar.Duration for one example of a full-featured library that is far more likely to be correct.","ref":"Moar.Duration.html","title":"Moar.Duration","type":"module"},{"doc":"Returns the duration between datetime and now, in the largest possible unit. datetime can be an ISO8601-formatted string, a DateTime , or a NaiveDateTime . See also from_now/1 . iex&gt; DateTime . utc_now ( ) ...&gt; |&gt; Moar.DateTime . subtract ( { 121 , :minute } ) ...&gt; |&gt; Moar.Duration . ago ( ) ...&gt; |&gt; Moar.Duration . shift ( :minute ) { 121 , :minute }","ref":"Moar.Duration.html#ago/1","title":"Moar.Duration.ago/1","type":"function"},{"doc":"Shifts duration to an approximately equal duration that's simpler. For example, {121, :second} would get shifted to {2, :minute} . Warning This function is lossy because it intentionally loses precision. If the time value of the duration is exactly 1, the duration is returned unchanged: {1, :minute} =&gt; {1, :minute} . Otherwise, the duration is shifted to the highest unit where the time value is &gt;= 2. iex&gt; Moar.Duration . approx ( { 1 , :minute } ) { 1 , :minute } iex&gt; Moar.Duration . approx ( { 7300 , :second } ) { 2 , :hour }","ref":"Moar.Duration.html#approx/1","title":"Moar.Duration.approx/1","type":"function"},{"doc":"Returns the duration between earlier and later , in the largest possible unit. earlier and later can be ISO8601-formatted strings, DateTime s, or NaiveDateTime s. iex&gt; earlier = ~U[2020-01-01T00:00:00.000000Z] iex&gt; later = ~U[2020-01-01T02:01:00.000000Z] iex&gt; Moar.Duration . between ( earlier , later ) { 121 , :minute }","ref":"Moar.Duration.html#between/2","title":"Moar.Duration.between/2","type":"function"},{"doc":"Converts a {duration, time_unit} tuple into a numeric duration, rounding down to the nearest whole number. Warning This function is lossy because it rounds down to the nearest whole number. Uses System.convert_time_unit/3 under the hood; see its documentation for more details. It is similar to shift/1 but this function returns an integer value, while shift/1 returns a duration tuple. iex&gt; Moar.Duration . convert ( { 121 , :second } , :minute ) 2","ref":"Moar.Duration.html#convert/2","title":"Moar.Duration.convert/2","type":"function"},{"doc":"Formats a duration in either a long or short style, with optional transformers and an optional suffix. The first argument is a duration tuple, unless one of the transformers is :ago , in which case it can be a DateTime , NaiveDateTime , or an ISO8601-formatted string. The second argument is optional and is the style, transformer, list of transformers, or suffix. The third argument is optional and is the transformer, list of transformers, or suffix. The fourth argument is optional and is the suffix. Styles: :long , which formats like &quot;25 seconds&quot; . :short , which formats like &quot;25s&quot; . Defaults to :long Transformers: :ago transforms via ago/1 :approx transforms via approx/1 :humanize transforms via humanize/1 If no transformers are specified, no transformations are applied. Suffix: A string that will be appended to the formatted result. If the :ago transformer is specified and a suffix is not specified, the suffix will default to &quot;ago&quot; . To use the &quot;ago&quot; transformer with no suffix, specify an empty string as the suffix ( nil will not suffice). iex&gt; Moar.Duration . format ( { 1 , :second } ) &quot;1 second&quot; iex&gt; Moar.Duration . format ( { 120 , :second } ) &quot;120 seconds&quot; iex&gt; Moar.Duration . format ( { 120 , :second } , :long ) &quot;120 seconds&quot; iex&gt; Moar.Duration . format ( { 120 , :second } , :short ) &quot;120s&quot; iex&gt; Moar.Duration . format ( { 120 , :second } , &quot;yonder&quot; ) &quot;120 seconds yonder&quot; iex&gt; Moar.Duration . format ( { 120 , :second } , :humanize ) &quot;2 minutes&quot; iex&gt; Moar.Duration . format ( { 120 , :second } , :humanize , &quot;yonder&quot; ) &quot;2 minutes yonder&quot; iex&gt; Moar.Duration . format ( { 310 , :second } ) &quot;310 seconds&quot; iex&gt; Moar.Duration . format ( { 310 , :second } , :approx ) &quot;5 minutes&quot; iex&gt; DateTime . utc_now ( ) ...&gt; |&gt; Moar.DateTime . add ( { - 310 , :second } ) ...&gt; |&gt; Moar.Duration . format ( :short , [ :ago , :approx ] , &quot;henceforth&quot; ) &quot;5m henceforth&quot;","ref":"Moar.Duration.html#format/4","title":"Moar.Duration.format/4","type":"function"},{"doc":"Returns the duration between now and datetime , in the largest possible unit. datetime can be an ISO8601-formatted string, a DateTime , or a NaiveDateTime . See also ago/1 . iex&gt; DateTime . utc_now ( ) ...&gt; |&gt; Moar.DateTime . add ( { 121 , :minute } ) ...&gt; |&gt; Moar.Duration . from_now ( ) ...&gt; |&gt; Moar.Duration . approx ( ) { 2 , :hour }","ref":"Moar.Duration.html#from_now/1","title":"Moar.Duration.from_now/1","type":"function"},{"doc":"If possible, shifts duration to a higher time unit that is more readable to a human. Returns duration unchanged if it cannot be exactly shifted. iex&gt; Moar.Duration . humanize ( { 60000 , :millisecond } ) { 1 , :minute } iex&gt; Moar.Duration . humanize ( { 48 , :hour } ) { 2 , :day } iex&gt; Moar.Duration . humanize ( { 49 , :hour } ) { 49 , :hour }","ref":"Moar.Duration.html#humanize/1","title":"Moar.Duration.humanize/1","type":"function"},{"doc":"Shifts duration to time_unit . It is similar to convert/1 but this function returns a duration tuple, while convert/1 just returns an integer value. Warning This function is lossy because it rounds down to the nearest whole number. iex&gt; Moar.Duration . shift ( { 121 , :second } , :minute ) { 2 , :minute }","ref":"Moar.Duration.html#shift/2","title":"Moar.Duration.shift/2","type":"function"},{"doc":"Shifts duration to the next smaller unit. Raises if it's already at the smallest unit (nanosecond). iex&gt; Moar.Duration . shift_down ( { 1 , :hour } ) { 60 , :minute }","ref":"Moar.Duration.html#shift_down/1","title":"Moar.Duration.shift_down/1","type":"function"},{"doc":"Shifts duration to the next larger unit. Raises if it's already at the largest unit (approx_year). Rounds the result towards zero. Warning This function is lossy because it rounds down to the nearest whole number. iex&gt; Moar.Duration . shift_up ( { 60 , :minute } ) { 1 , :hour } iex&gt; Moar.Duration . shift_up ( { 125 , :minute } ) { 2 , :hour }","ref":"Moar.Duration.html#shift_up/1","title":"Moar.Duration.shift_up/1","type":"function"},{"doc":"Shortcut to format(duration, :long) . See format/4 .","ref":"Moar.Duration.html#to_string/1","title":"Moar.Duration.to_string/1","type":"function"},{"doc":"","ref":"Moar.Duration.html#t:date_time_ish/0","title":"Moar.Duration.date_time_ish/0","type":"type"},{"doc":"","ref":"Moar.Duration.html#t:format_style/0","title":"Moar.Duration.format_style/0","type":"type"},{"doc":"","ref":"Moar.Duration.html#t:format_transformer/0","title":"Moar.Duration.format_transformer/0","type":"type"},{"doc":"","ref":"Moar.Duration.html#t:format_transformers/0","title":"Moar.Duration.format_transformers/0","type":"type"},{"doc":"","ref":"Moar.Duration.html#t:t/0","title":"Moar.Duration.t/0","type":"type"},{"doc":"","ref":"Moar.Duration.html#t:time_unit/0","title":"Moar.Duration.time_unit/0","type":"type"},{"doc":"Enum-related functions.","ref":"Moar.Enum.html","title":"Moar.Enum","type":"module"},{"doc":"Like Enum.at/3 but raises if index is out of bounds.","ref":"Moar.Enum.html#at!/3","title":"Moar.Enum.at!/3","type":"function"},{"doc":"Removes nil elements from enum .","ref":"Moar.Enum.html#compact/1","title":"Moar.Enum.compact/1","type":"function"},{"doc":"Returns the first item of enum , or raises if it is empty.","ref":"Moar.Enum.html#first!/1","title":"Moar.Enum.first!/1","type":"function"},{"doc":"Returns true if the value is a map or a keyword list. This cannot be used as a guard because it uses Keyword.keyword? under the hood. Also, because of that, it might scan an entire list to see if it's a keyword list, so it might be expensive.","ref":"Moar.Enum.html#is_map_or_keyword/1","title":"Moar.Enum.is_map_or_keyword/1","type":"function"},{"doc":"Sorts enum case-insensitively. Uses Enum.sort_by/3 under the hood.","ref":"Moar.Enum.html#isort/1","title":"Moar.Enum.isort/1","type":"function"},{"doc":"Sorts enum case-insensitively by mapper function. Uses Enum.sort_by/3 under the hood.","ref":"Moar.Enum.html#isort_by/2","title":"Moar.Enum.isort_by/2","type":"function"},{"doc":"Returns a list of elements at the given indices. If :all is given instead of a list of indices, the entire enum is returned. iex&gt; Moar.Enum . take_at ( [ &quot;A&quot; , &quot;B&quot; , &quot;C&quot; ] , [ 0 , 2 ] ) [ &quot;A&quot; , &quot;C&quot; ] iex&gt; Moar.Enum . take_at ( [ &quot;A&quot; , &quot;B&quot; , &quot;C&quot; ] , :all ) [ &quot;A&quot; , &quot;B&quot; , &quot;C&quot; ]","ref":"Moar.Enum.html#take_at/2","title":"Moar.Enum.take_at/2","type":"function"},{"doc":"Returns :tid fields from enumerable . This unusual function exists because the authors of Moar use tids (test IDs) extensively in tests.","ref":"Moar.Enum.html#tids/1","title":"Moar.Enum.tids/1","type":"function"},{"doc":"File-related functions.","ref":"Moar.File.html","title":"Moar.File","type":"module"},{"doc":"Returns a randomly-named path in the system temp directory. Does not actually create the file at the path. For example, Moar.File.new_tempfile_path(&quot;.txt&quot;) would return something like &quot;/var/folders/3r/n09zyyy16yddj9m2d1ppzvt40000gn/T/XY7V5E26DV.txt&quot; . iex&gt; Moar.File . new_tempfile_path ( &quot;.txt&quot; ) |&gt; File . exists? ( ) false","ref":"Moar.File.html#new_tempfile_path/1","title":"Moar.File.new_tempfile_path/1","type":"function"},{"doc":"Writes contents to a new temp file with extension file_extension , and returns the path to the file. For example, Moar.File.write_tempfile(&quot;hello&quot;, &quot;.txt&quot;) would create a file in a path something like /var/folders/3r/n09zyyy16yddj9m2d1ppzvt40000gn/T/RHUEJCCH22.txt with the contents hello . iex&gt; Moar.File . write_tempfile ( &quot;hello&quot; , &quot;.txt&quot; ) |&gt; File . read! ( ) &quot;hello&quot;","ref":"Moar.File.html#write_tempfile/2","title":"Moar.File.write_tempfile/2","type":"function"},{"doc":"Map-related functions.","ref":"Moar.Map.html","title":"Moar.Map","type":"module"},{"doc":"Converts key in map to an atom, optionally transforming the value with value_transformer . Raises if key is a string and map already has an atomized version of that key. iex&gt; Moar.Map . atomize_key ( %{ &quot;number-one&quot; =&gt; &quot;one&quot; , &quot;number-two&quot; =&gt; &quot;two&quot; } , &quot;number-one&quot; ) %{ :number_one =&gt; &quot;one&quot; , &quot;number-two&quot; =&gt; &quot;two&quot; } iex&gt; Moar.Map . atomize_key ( %{ &quot;number-one&quot; =&gt; &quot;one&quot; , &quot;number-two&quot; =&gt; &quot;two&quot; } , &quot;number-one&quot; , &amp; String . upcase / 1 ) %{ :number_one =&gt; &quot;ONE&quot; , &quot;number-two&quot; =&gt; &quot;two&quot; }","ref":"Moar.Map.html#atomize_key/3","title":"Moar.Map.atomize_key/3","type":"function"},{"doc":"Like atomize_key/3 but raises if key is not in map .","ref":"Moar.Map.html#atomize_key!/3","title":"Moar.Map.atomize_key!/3","type":"function"},{"doc":"Converts keys in map to atoms. Raises if converting a key from a string to an atom would result in a key conflict. iex&gt; Moar.Map . atomize_keys ( %{ &quot;a&quot; =&gt; 1 , &quot;b&quot; =&gt; 2 } ) %{ a : 1 , b : 2 }","ref":"Moar.Map.html#atomize_keys/1","title":"Moar.Map.atomize_keys/1","type":"function"},{"doc":"Converts keys to atoms, traversing through descendant lists and maps. Raises if converting a key from a string to an atom would result in a key conflict. iex&gt; Moar.Map . deep_atomize_keys ( %{ &quot;a&quot; =&gt; %{ &quot;aa&quot; =&gt; 1 } , &quot;b&quot; =&gt; [ %{ &quot;bb&quot; =&gt; 2 } , %{ &quot;bbb&quot; =&gt; 3 } ] } ) %{ a : %{ aa : 1 } , b : [ %{ bb : 2 } , %{ bbb : 3 } ] }","ref":"Moar.Map.html#deep_atomize_keys/1","title":"Moar.Map.deep_atomize_keys/1","type":"function"},{"doc":"Deeply merges two maps into a single map. (It will also accept keyword lists and convert them to maps.) Optionally accepts conflict_fn which gets called when both enumerables have values at the same keypath. It receives the conflicting values from each map and is expected to return the winning value. iex&gt; Moar.Map . deep_merge ( %{ fruit : %{ apples : 3 , bananas : 5 } , veggies : %{ carrots : 10 } } , [ fruit : [ cherries : 20 ] ] ) %{ fruit : %{ apples : 3 , bananas : 5 , cherries : 20 } , veggies : %{ carrots : 10 } } iex&gt; Moar.Map . deep_merge ( %{ a : %{ b : 1 } } , %{ a : %{ b : 2 } } ) %{ a : %{ b : 2 } } iex&gt; Moar.Map . deep_merge ( %{ a : %{ b : 1 } } , %{ a : %{ b : 2 } } , fn x , _y -&gt; x end ) %{ a : %{ b : 1 } } iex&gt; Moar.Map . deep_merge ( %{ a : %{ b : 1 } } , %{ a : %{ b : 2 } } , fn x , y -&gt; [ x , y ] end ) %{ a : %{ b : [ 1 , 2 ] } }","ref":"Moar.Map.html#deep_merge/3","title":"Moar.Map.deep_merge/3","type":"function"},{"doc":"Merges two enumerables into a single map. iex&gt; Moar.Map . merge ( %{ a : 1 } , [ b : 2 ] ) %{ a : 1 , b : 2 }","ref":"Moar.Map.html#merge/2","title":"Moar.Map.merge/2","type":"function"},{"doc":"Puts a key/value pair into the given map if the key is not alredy in the map, or if the value in the map is blank as defined by Moar.Term.blank?/1 . Also, the map parameter can be any enumerable that can be turned into a map via Enum.into/2 . iex&gt; %{ a : 1 } |&gt; Moar.Map . put_if_blank ( :b , 2 ) %{ a : 1 , b : 2 } iex&gt; %{ a : 1 , b : nil } |&gt; Moar.Map . put_if_blank ( :b , 2 ) %{ a : 1 , b : 2 } iex&gt; %{ a : 1 , b : 3 } |&gt; Moar.Map . put_if_blank ( :b , 2 ) %{ a : 1 , b : 3 }","ref":"Moar.Map.html#put_if_blank/3","title":"Moar.Map.put_if_blank/3","type":"function"},{"doc":"Returns a copy of map with old_key_name changed to new_key_name . old_key_name and new_key_name are passed in as a {old_key_name, new_key_name} tuple. iex&gt; %{ &quot;color&quot; =&gt; &quot;red&quot; , &quot;size&quot; =&gt; &quot;medium&quot; } |&gt; Moar.Map . rename_key ( { &quot;color&quot; , &quot;colour&quot; } ) %{ &quot;colour&quot; =&gt; &quot;red&quot; , &quot;size&quot; =&gt; &quot;medium&quot; }","ref":"Moar.Map.html#rename_key/2","title":"Moar.Map.rename_key/2","type":"function"},{"doc":"Returns a copy of map with old_key_name changed to new_key_name . iex&gt; %{ &quot;color&quot; =&gt; &quot;red&quot; , &quot;size&quot; =&gt; &quot;medium&quot; } |&gt; Moar.Map . rename_key ( &quot;color&quot; , &quot;colour&quot; ) %{ &quot;colour&quot; =&gt; &quot;red&quot; , &quot;size&quot; =&gt; &quot;medium&quot; }","ref":"Moar.Map.html#rename_key/3","title":"Moar.Map.rename_key/3","type":"function"},{"doc":"Like rename_key/2 but raises if key is not in map","ref":"Moar.Map.html#rename_key!/2","title":"Moar.Map.rename_key!/2","type":"function"},{"doc":"Like rename_key/3 but raises if key is not in map","ref":"Moar.Map.html#rename_key!/3","title":"Moar.Map.rename_key!/3","type":"function"},{"doc":"Returns a copy of map after changing key names supplied by keys_map . %{ &quot;behavior&quot; =&gt; &quot;chill&quot; , &quot;color&quot; =&gt; &quot;red&quot; } |&gt; Moar.Map . rename_keys ( %{ &quot;behavior&quot; =&gt; &quot;behaviour&quot; , &quot;color&quot; =&gt; &quot;colour&quot; } ) %{ &quot;behaviour&quot; =&gt; &quot;chill&quot; , &quot;colour&quot; =&gt; &quot;red&quot; }","ref":"Moar.Map.html#rename_keys/2","title":"Moar.Map.rename_keys/2","type":"function"},{"doc":"Like rename_keys/2 but raises if any key in keys_map is not in map .","ref":"Moar.Map.html#rename_keys!/2","title":"Moar.Map.rename_keys!/2","type":"function"},{"doc":"Converts keys in map to strings. iex&gt; Moar.Map . stringify_keys ( %{ a : 1 , b : 2 } ) %{ &quot;a&quot; =&gt; 1 , &quot;b&quot; =&gt; 2 }","ref":"Moar.Map.html#stringify_keys/1","title":"Moar.Map.stringify_keys/1","type":"function"},{"doc":"Transforms values of map using transformer function. iex&gt; %{ &quot;foo&quot; =&gt; &quot;chicken&quot; , &quot;bar&quot; =&gt; &quot;cow&quot; , &quot;baz&quot; =&gt; &quot;pig&quot; } |&gt; Moar.Map . transform ( &quot;foo&quot; , &amp; String . upcase / 1 ) %{ &quot;foo&quot; =&gt; &quot;CHICKEN&quot; , &quot;bar&quot; =&gt; &quot;cow&quot; , &quot;baz&quot; =&gt; &quot;pig&quot; } iex&gt; %{ &quot;foo&quot; =&gt; &quot;chicken&quot; , &quot;bar&quot; =&gt; &quot;cow&quot; , &quot;baz&quot; =&gt; &quot;pig&quot; } |&gt; Moar.Map . transform ( [ &quot;foo&quot; , &quot;bar&quot; ] , &amp; String . upcase / 1 ) %{ &quot;foo&quot; =&gt; &quot;CHICKEN&quot; , &quot;bar&quot; =&gt; &quot;COW&quot; , &quot;baz&quot; =&gt; &quot;pig&quot; }","ref":"Moar.Map.html#transform/3","title":"Moar.Map.transform/3","type":"function"},{"doc":"NaiveDateTime-related functions. See also Moar.DateTime .","ref":"Moar.NaiveDateTime.html","title":"Moar.NaiveDateTime","type":"module"},{"doc":"Like NaiveDateTime.add/2 but takes a Moar.Duration . See also subtract/1 and Moar.DateTime.add/2 . Note This function is naive and intentionally doesn't account for real-world calendars and all of their complexity, such as leap years, leap days, daylight saving time, past and future calendar oddities, etc. As &quot;Falsehoods programmers believe about time&quot; says, &quot;If you think you understand everything about time, you're probably doing it wrong.&quot; See Cldr.Calendar.plus/2 for one example of a function that is far more likely to be correct. iex&gt; start = ~N[2022-01-01T00:00:00] iex&gt; Moar.NaiveDateTime . add ( start , { 3 , :minute } ) ~N[2022-01-01T00:03:00]","ref":"Moar.NaiveDateTime.html#add/2","title":"Moar.NaiveDateTime.add/2","type":"function"},{"doc":"Like NaiveDateTime.from_iso8601/1 but raises if the string cannot be parsed. iex&gt; Moar.NaiveDateTime . from_iso8601! ( &quot;2022-01-01T00:00:00&quot; ) ~N[2022-01-01T00:00:00] iex&gt; Moar.NaiveDateTime . from_iso8601! ( &quot;2022-01-01T00:00:00+0800&quot; ) ~N[2022-01-01T00:00:00] iex&gt; Moar.NaiveDateTime . from_iso8601! ( &quot;Next Thursday after lunch&quot; ) ** (ArgumentError) Invalid ISO8601 format: &quot;Next Thursday after lunch&quot;","ref":"Moar.NaiveDateTime.html#from_iso8601!/1","title":"Moar.NaiveDateTime.from_iso8601!/1","type":"function"},{"doc":"Subtracts duration from date_time . See also add/1 and Moar.DateTime.subtract/2 . Note This function is naive and intentionally doesn't account for real-world calendars and all of their complexity, such as leap years, leap days, daylight saving time, past and future calendar oddities, etc. As &quot;Falsehoods programmers believe about time&quot; says, &quot;If you think you understand everything about time, you're probably doing it wrong.&quot; See Cldr.Calendar.minus/4 for one example of a function that is far more likely to be correct. iex&gt; start = ~N[2022-01-01T00:03:00] iex&gt; Moar.NaiveDateTime . subtract ( start , { 3 , :minute } ) ~N[2022-01-01T00:00:00]","ref":"Moar.NaiveDateTime.html#subtract/2","title":"Moar.NaiveDateTime.subtract/2","type":"function"},{"doc":"Like NaiveDateTime.to_iso8601/1 but rounds to the nearest second first. iex&gt; Moar.NaiveDateTime . to_iso8601_rounded ( ~N[2022-01-01T01:02:03.456789] ) &quot;2022-01-01T01:02:03&quot;","ref":"Moar.NaiveDateTime.html#to_iso8601_rounded/1","title":"Moar.NaiveDateTime.to_iso8601_rounded/1","type":"function"},{"doc":"Returns the current UTC time plus or minus the given duration. iex&gt; Moar.NaiveDateTime . utc_now ( plus : { 10500 , :millisecond } ) ...&gt; |&gt; Moar.Duration . format ( [ :approx , :from_now ] ) &quot;10 seconds from now&quot; iex&gt; Moar.NaiveDateTime . utc_now ( minus : { 10 , :second } ) ...&gt; |&gt; Moar.Duration . format ( [ :approx , :ago ] ) &quot;10 seconds ago&quot;","ref":"Moar.NaiveDateTime.html#utc_now/1","title":"Moar.NaiveDateTime.utc_now/1","type":"function"},{"doc":"Extracts keys and values from enumerables, especially from function options. There are two main functions, each of which takes an opts enumerable as input. get/3 extracts one value from the opts with an optional default value. take/2 extracts multiple values from the opts with optional default values for some or all keys. get/3 and take/2 differ from their Map and Keyword counterparts in the following ways: get/3 and take/2 accept any enumerable, including maps and keyword lists. get/3 and take/2 will fall back to the default value if the given key's value is blank as defined by Moar.Term.blank?/1 ( nil , empty strings, strings made up only of whitespace, empty lists, and empty maps). The corresponding Map and Keyword functions only fall back to the default value if the value is exactly nil . take/2 allows default values to be specified. take/2 will return the value for a requested key even if the key is not in the input enumerable. Example using get/2 and get/3 : def build_url ( path , opts \\\\ [ ] ) do % URI { path : path , host : Moar.Opts . get ( opts , :host , &quot;localhost&quot; ) , port : Moar.Opts . get ( opts , :port ) , scheme : &quot;https&quot; } |&gt; URI . to_string ( ) end Examples using take/2 : # example using pattern matching def build_url ( path , opts \\ [ ] ) do %{ host : h , port : p } = Moar.Opts . take ( opts , [ :port , host : &quot;localhost&quot; ] ) % URI { path : path , host : h , port : p , scheme : &quot;https&quot; } |&gt; URI . to_string ( ) end # example rebinding `opts` to the parsed opts def build_url ( path , opts \\ [ ] ) do opts = Moar.Opts . take ( opts , [ :port , host : &quot;localhost&quot; ] ) % URI { path : path , host : opts . host , port : opts . port , scheme : &quot;https&quot; } |&gt; URI . to_string ( ) end","ref":"Moar.Opts.html","title":"Moar.Opts","type":"module"},{"doc":"Get the value of key from input , falling back to optional default if the key does not exist, or if its value is blank (via Moar.Term.blank?/1 ). iex&gt; [ a : 1 , b : 2 ] |&gt; Moar.Opts . get ( :a ) 1 iex&gt; [ a : 1 , b : 2 ] |&gt; Moar.Opts . get ( :c ) nil iex&gt; [ a : 1 , b : 2 , c : &quot;&quot; ] |&gt; Moar.Opts . get ( :c ) nil iex&gt; [ a : 1 , b : 2 , c : %{ } ] |&gt; Moar.Opts . get ( :c ) nil iex&gt; [ a : 1 , b : 2 , c : &quot; &quot; ] |&gt; Moar.Opts . get ( :c , 300 ) 300","ref":"Moar.Opts.html#get/3","title":"Moar.Opts.get/3","type":"function"},{"doc":"Get the value each key in keys from input , falling back to optional default values for keys that do not exist, or for values that are blank (via Moar.Term.blank?/1 ). If key does not exist in keys , return nil , or return the default value if provided. keys is a list of keys (e.g., [:a, :b] ), a keyword list of keys and default values (e.g., [a: 1, b: 2] ), or a hybrid list/keyword list (e.g., [:a, b: 2] ) iex&gt; [ a : 1 , b : 2 ] |&gt; Moar.Opts . take ( [ :a , :c ] ) %{ a : 1 , c : nil } iex&gt; [ a : 1 , b : 2 ] |&gt; Moar.Opts . take ( [ :a , b : 0 , c : 3 ] ) %{ a : 1 , b : 2 , c : 3 }","ref":"Moar.Opts.html#take/2","title":"Moar.Opts.take/2","type":"function"},{"doc":"Protocol-related functions.","ref":"Moar.Protocol.html","title":"Moar.Protocol","type":"module"},{"doc":"Returns x or raises if x does not implment protocol . iex&gt; Moar.Protocol . implements! ( ~D[2000-01-02] , String.Chars ) ~D[2000-01-02]","ref":"Moar.Protocol.html#implements!/2","title":"Moar.Protocol.implements!/2","type":"function"},{"doc":"Returns true if x implements protocol . iex&gt; Moar.Protocol . implements? ( ~D[2000-01-02] , String.Chars ) true","ref":"Moar.Protocol.html#implements?/2","title":"Moar.Protocol.implements?/2","type":"function"},{"doc":"Generates random data.","ref":"Moar.Random.html","title":"Moar.Random","type":"module"},{"doc":"Returns a random integer between 0 and max .","ref":"Moar.Random.html#integer/1","title":"Moar.Random.integer/1","type":"function"},{"doc":"Returns a base64- or base32-encoded random string of 32 characters. See Moar.Random.string/2 .","ref":"Moar.Random.html#string/1","title":"Moar.Random.string/1","type":"function"},{"doc":"Returns a base64- or base32-encoded random string of given length. iex&gt; Moar.Random . string ( ) &quot;Sr/y4m/YiVSJcIgI5lG+76vMfaZ7KZ7c&quot; iex&gt; Moar.Random . string ( 5 ) &quot;9pJrK&quot; iex&gt; Moar.Random . string ( 5 , :base32 ) &quot;AC53Z&quot;","ref":"Moar.Random.html#string/2","title":"Moar.Random.string/2","type":"function"},{"doc":"","ref":"Moar.Random.html#t:encoding/0","title":"Moar.Random.encoding/0","type":"type"},{"doc":"Retryable functions. These functions are particularly useful in higher-level tests, though they can be used for any kind of waiting. See also the retry Hex package. One use of rescue_for/3 in a test is to wait until an assertion succeeds: # wait up to 5 seconds for the element with ID &quot;status&quot; to be &quot;finished&quot; Moar.Retry . rescue_for! ( 5000 , fn -&gt; assert view |&gt; element ( &quot;#status&quot; ) |&gt; render ( ) == &quot;finished&quot; end ) One use of retry_for/3 in a test is to wait until something changes before making an assertion: Moar.Retry . retry_for ( 5000 , fn -&gt; view |&gt; element ( &quot;#status&quot; ) |&gt; render ( ) == &quot;finished&quot; end ) assert view |&gt; element ( &quot;#total&quot; ) |&gt; render ( ) == &quot;8,675,309&quot;","ref":"Moar.Retry.html","title":"Moar.Retry","type":"module"},{"doc":"Run fun every interval_ms until it doesn't raise an exception. If timeout expires, the exception will be re-raised. timeout can be an integer in milliseconds or a Moar.Duration tuple iex&gt; Moar.Retry . rescue_for! ( 20 , fn -&gt; raise &quot;always fails&quot; end , 2 ) ** (RuntimeError) always fails iex&gt; Moar.Retry . rescue_for! ( { 20 , :millisecond } , fn -&gt; raise &quot;always fails&quot; end , 2 ) ** (RuntimeError) always fails","ref":"Moar.Retry.html#rescue_for!/3","title":"Moar.Retry.rescue_for!/3","type":"function"},{"doc":"Run fun every interval_ms until it doesn't raise an exception. If the current time reaches expiry , the exception will be re-raised. iex&gt; date_time = DateTime . add ( DateTime . utc_now ( ) , 20 , :millisecond ) iex&gt; Moar.Retry . rescue_until! ( date_time , fn -&gt; raise &quot;always fails&quot; end , 2 ) ** (RuntimeError) always fails","ref":"Moar.Retry.html#rescue_until!/3","title":"Moar.Retry.rescue_until!/3","type":"function"},{"doc":"Run fun every interval_ms until it returns a truthy value, returning {:ok, &lt;value&gt;} . If timeout expires, returns {:error, :timeout} . timeout can be an integer in milliseconds or a Moar.Duration tuple iex&gt; Moar.Retry . retry_for ( 20 , fn -&gt; 10 end , 2 ) { :ok , 10 } iex&gt; Moar.Retry . retry_for ( 20 , fn -&gt; false end , 2 ) { :error , :timeout }","ref":"Moar.Retry.html#retry_for/3","title":"Moar.Retry.retry_for/3","type":"function"},{"doc":"Run fun every interval_ms until it returns a truthy value, returning {:ok, &lt;value&gt;} . If the current time reaches expiry , returns {:error, :timeout} . iex&gt; date_time = DateTime . add ( DateTime . utc_now ( ) , 20 , :millisecond ) iex&gt; Moar.Retry . retry_until ( date_time , fn -&gt; false end , 2 ) { :error , :timeout }","ref":"Moar.Retry.html#retry_until/3","title":"Moar.Retry.retry_until/3","type":"function"},{"doc":"String-related functions.","ref":"Moar.String.html","title":"Moar.String","type":"module"},{"doc":"Appends suffix to string unless string is blank according to Moar.Term.blank?/1 . iex&gt; Moar.String . append_unless_blank ( &quot;foo&quot; , &quot;-bar&quot; ) &quot;foo-bar&quot; iex&gt; Moar.String . append_unless_blank ( &quot;&quot; , &quot;-bar&quot; ) &quot;&quot; iex&gt; Moar.String . append_unless_blank ( nil , &quot;-bar&quot; ) nil","ref":"Moar.String.html#append_unless_blank/2","title":"Moar.String.append_unless_blank/2","type":"function"},{"doc":"Dasherizes term . A shortcut to slug(term, &quot;-&quot;) . See docs for slug/2 . iex&gt; Moar.String . dasherize ( &quot;foo bar&quot; ) &quot;foo-bar&quot;","ref":"Moar.String.html#dasherize/1","title":"Moar.String.dasherize/1","type":"function"},{"doc":"Truncate s to max_length by replacing the middle of the string with replacement , which defaults to the single unicode character … . Note that the final length of the string will be max_length plus the length of replacement . iex&gt; Moar.String . inner_truncate ( &quot;abcdefghijklmnopqrstuvwxyz&quot; , 10 ) &quot;abcde…vwxyz&quot; iex&gt; Moar.String . inner_truncate ( &quot;abcdefghijklmnopqrstuvwxyz&quot; , 10 , &quot;&lt;==&gt;&quot; ) &quot;abcde&lt;==&gt;vwxyz&quot;","ref":"Moar.String.html#inner_truncate/3","title":"Moar.String.inner_truncate/3","type":"function"},{"doc":"Pluralizes a string. When count is -1 or 1, returns the second argument (the singular string). Otherwise, returns the third argument (the pluralized string), or if the third argument is a function, calls the function with the singular string as an argument. iex&gt; Moar.String . pluralize ( 1 , &quot;fish&quot; , &quot;fishies&quot; ) &quot;fish&quot; iex&gt; Moar.String . pluralize ( 2 , &quot;fish&quot; , &quot;fishies&quot; ) &quot;fishies&quot; iex&gt; Moar.String . pluralize ( 2 , &quot;fish&quot; , fn singular -&gt; singular &lt;&gt; &quot;ies&quot; end ) &quot;fishies&quot; iex&gt; Moar.String . pluralize ( 2 , &quot;fish&quot; , &amp; ( &amp;1 &lt;&gt; &quot;ies&quot; ) ) &quot;fishies&quot;","ref":"Moar.String.html#pluralize/3","title":"Moar.String.pluralize/3","type":"function"},{"doc":"Compares the two binaries in constant-time to avoid timing attacks. See: http://codahale.com/a-lesson-in-timing-attacks/ . iex&gt; Moar.String . secure_compare ( &quot;foo&quot; , &quot;bar&quot; ) false","ref":"Moar.String.html#secure_compare/2","title":"Moar.String.secure_compare/2","type":"function"},{"doc":"Creates slugs like foo-bar-123 or foo_bar from various input types. Converts strings, atoms, and anything else that implements String.Chars , plus lists of those things, to a single string after removing non-alphanumeric characters, and then joins them with joiner . Existing occurrences of joiner are kept, including leading and trailing ones. dasherize/1 and underscore/1 are shortcuts that specify a joiner. iex&gt; Moar.String . slug ( &quot;foo bar&quot; , &quot;_&quot; ) &quot;foo_bar&quot; iex&gt; Moar.String . slug ( &quot;foo bar&quot; , &quot;+&quot; ) &quot;foo+bar&quot; iex&gt; Moar.String . slug ( [ &quot;foo&quot; , &quot;bar&quot; ] , &quot;+&quot; ) &quot;foo+bar&quot; iex&gt; Moar.String . slug ( &quot;_foo bar&quot; , &quot;_&quot; ) &quot;_foo_bar&quot; iex&gt; [ &quot;foo&quot; , &quot;FOO&quot; , :foo ] |&gt; Enum . map ( &amp; Moar.String . slug ( &amp;1 , &quot;-&quot; ) ) [ &quot;foo&quot; , &quot;foo&quot; , &quot;foo&quot; ] iex&gt; [ &quot;foo-bar&quot; , &quot;foo_bar&quot; , :foo_bar , &quot; fooBar &quot; , &quot; ?foo ! bar &quot; ] |&gt; Enum . map ( &amp; Moar.String . slug ( &amp;1 , &quot;-&quot; ) ) [ &quot;foo-bar&quot; , &quot;foo-bar&quot; , &quot;foo-bar&quot; , &quot;foo-bar&quot; , &quot;foo-bar&quot; ]","ref":"Moar.String.html#slug/2","title":"Moar.String.slug/2","type":"function"},{"doc":"Trims a string and replaces consecutive whitespace characters with a single space. iex&gt; Moar.String . squish ( &quot; foo bar baz &quot; ) &quot;foo bar baz&quot;","ref":"Moar.String.html#squish/1","title":"Moar.String.squish/1","type":"function"},{"doc":"Adds surrounder to the beginning and end of s . iex&gt; Moar.String . surround ( &quot;Hello&quot; , &quot;**&quot; ) &quot;**Hello**&quot;","ref":"Moar.String.html#surround/2","title":"Moar.String.surround/2","type":"function"},{"doc":"Adds prefix to the beginning of s and suffix to the end. iex&gt; Moar.String . surround ( &quot;Hello&quot; , &quot;“&quot; , &quot;”&quot; ) &quot;“Hello”&quot;","ref":"Moar.String.html#surround/3","title":"Moar.String.surround/3","type":"function"},{"doc":"Change the case of a string. iex&gt; Moar.String . to_case ( &quot;text_with_case&quot; , :camel_case ) &quot;TextWithCase&quot; iex&gt; Moar.String . to_case ( &quot;textWithCase&quot; , :camel_case ) &quot;TextWithCase&quot; iex&gt; Moar.String . to_case ( &quot;some random text&quot; , :camel_case ) &quot;SomeRandomText&quot; iex&gt; Moar.String . to_case ( &quot;text_with_case&quot; , :lower_camel_case ) &quot;textWithCase&quot; iex&gt; Moar.String . to_case ( &quot;textWithCase&quot; , :lower_camel_case ) &quot;textWithCase&quot; iex&gt; Moar.String . to_case ( &quot;some random text&quot; , :lower_camel_case ) &quot;someRandomText&quot; iex&gt; Moar.String . to_case ( &quot;text_with_case&quot; , :snake_case ) &quot;text_with_case&quot; iex&gt; Moar.String . to_case ( &quot;textWithCase&quot; , :snake_case ) &quot;text_with_case&quot; iex&gt; Moar.String . to_case ( &quot;some random text&quot; , :snake_case ) &quot;some_random_text&quot;","ref":"Moar.String.html#to_case/2","title":"Moar.String.to_case/2","type":"function"},{"doc":"Converts a string to an integer. Returns nil if the argument is nil or empty string. Returns the argument without complaint if it is already an integer. iex&gt; Moar.String . to_integer ( &quot;12,345&quot; ) 12_345 iex&gt; Moar.String . to_integer ( &quot;&quot; ) nil iex&gt; Moar.String . to_integer ( 12_345 ) 12_345","ref":"Moar.String.html#to_integer/1","title":"Moar.String.to_integer/1","type":"function"},{"doc":"Like to_integer/1 but with options: :lenient option removes non-digit characters first default: option specifies a default in case s is nil iex&gt; Moar.String . to_integer ( &quot;USD$25&quot; , :lenient ) 25 iex&gt; Moar.String . to_integer ( nil , default : 0 ) 0","ref":"Moar.String.html#to_integer/2","title":"Moar.String.to_integer/2","type":"function"},{"doc":"Like String.trim/1 but returns nil if the argument is nil.","ref":"Moar.String.html#trim/1","title":"Moar.String.trim/1","type":"function"},{"doc":"Truncates s at the last instance of at , causing the string to be at most limit characters. iex&gt; Moar.String . truncate_at ( &quot;I like apples. I like bananas. I like cherries.&quot; , &quot;.&quot; , 35 ) &quot;I like apples. I like bananas.&quot;","ref":"Moar.String.html#truncate_at/3","title":"Moar.String.truncate_at/3","type":"function"},{"doc":"Underscores term . A shortcut to slug(term, &quot;_&quot;) . See docs for slug/2 . iex&gt; Moar.String . underscore ( &quot;foo bar&quot; ) &quot;foo_bar&quot;","ref":"Moar.String.html#underscore/1","title":"Moar.String.underscore/1","type":"function"},{"doc":"","ref":"Moar.String.html#t:string_case/0","title":"Moar.String.string_case/0","type":"type"},{"doc":"Syntactic sugar functions. These functions are intended to be used by importing the functions or the whole module: import Moar.Sugar , only : [ noreply : 1 ] def handle_event ( &quot;foo&quot; , _params , socket ) do socket |&gt; assign ( foo : &quot;bar&quot; ) |&gt; noreply ( ) end","ref":"Moar.Sugar.html","title":"Moar.Sugar","type":"module"},{"doc":"Wraps a term in an :error tuple. Useful in pipelines. iex&gt; %{ } |&gt; Map . put ( :count , &quot;unknown&quot; ) |&gt; Moar.Sugar . error ( ) { :error , %{ count : &quot;unknown&quot; } }","ref":"Moar.Sugar.html#error/1","title":"Moar.Sugar.error/1","type":"function"},{"doc":"Unwraps an :error tuple, raising if the term is not an :error tuple. iex&gt; { :error , 1 } |&gt; Moar.Sugar . error! ( ) 1","ref":"Moar.Sugar.html#error!/1","title":"Moar.Sugar.error!/1","type":"function"},{"doc":"Wraps a term in a :noreply tuple. Useful in pipelines. iex&gt; %{ } |&gt; Map . put ( :count , 0 ) |&gt; Moar.Sugar . noreply ( ) { :noreply , %{ count : 0 } }","ref":"Moar.Sugar.html#noreply/1","title":"Moar.Sugar.noreply/1","type":"function"},{"doc":"Wraps a term in an :ok tuple. Useful in pipelines. iex&gt; %{ } |&gt; Map . put ( :count , 10 ) |&gt; Moar.Sugar . ok ( ) { :ok , %{ count : 10 } }","ref":"Moar.Sugar.html#ok/1","title":"Moar.Sugar.ok/1","type":"function"},{"doc":"Unwraps an :ok tuple, raising if the term is not an :ok tuple. iex&gt; { :ok , 1 } |&gt; Moar.Sugar . ok! ( ) 1","ref":"Moar.Sugar.html#ok!/1","title":"Moar.Sugar.ok!/1","type":"function"},{"doc":"Accepts two arguments and returns the second. Useful at the end of the pipeline when you want to return a different value than the last result of the pipeline, such as when the pipeline has side effects. iex&gt; %{ } |&gt; Map . put ( :count , 20 ) |&gt; Moar.Sugar . returning ( :count_updated ) :count_updated","ref":"Moar.Sugar.html#returning/2","title":"Moar.Sugar.returning/2","type":"function"},{"doc":"Blank/present functions for terms. A term is considered present when it is not blank. A term is considered blank when: it is nil it is false it is a string, and its length after being trimmed is 0 it is an empty list it is an empty map","ref":"Moar.Term.html","title":"Moar.Term","type":"module"},{"doc":"Returns true if the term is blank, nil, or empty. iex&gt; Moar.Term . blank? ( nil ) true iex&gt; Moar.Term . blank? ( &quot; &quot; ) true iex&gt; Moar.Term . blank? ( [ ] ) true iex&gt; Moar.Term . blank? ( %{ } ) true","ref":"Moar.Term.html#blank?/1","title":"Moar.Term.blank?/1","type":"function"},{"doc":"Returns the value if it is present (via present? ), or else returns the default value. iex&gt; Moar.Term . presence ( 20 , 100 ) 20 iex&gt; Moar.Term . presence ( nil , 100 ) 100","ref":"Moar.Term.html#presence/2","title":"Moar.Term.presence/2","type":"function"},{"doc":"Returns true if the term is not blank, nil, or empty. iex&gt; Moar.Term . present? ( 1 ) true iex&gt; Moar.Term . present? ( [ 1 ] ) true iex&gt; Moar.Term . present? ( %{ a : 1 } ) true iex&gt; Moar.Term . present? ( &quot;1&quot; ) true","ref":"Moar.Term.html#present?/1","title":"Moar.Term.present?/1","type":"function"},{"doc":"Tuple-related functions.","ref":"Moar.Tuple.html","title":"Moar.Tuple","type":"module"},{"doc":"Converts a list of tuples to a single tuple whose first element is the first element of each tuple in the list (which must all be the same), and whose second element is a list containing the second elements of each tuple in the list. Raises if the list contains tuples whose first elements are not all the same. iex&gt; Moar.Tuple . from_list! ( [ { :ok , :a } , { :ok , :b } ] ) { :ok , [ :a , :b ] } iex&gt; Moar.Tuple . from_list! ( [ { :a , 1 } , { :a , 2 } , { :a , 3 } ] ) { :a , [ 1 , 2 , 3 ] } iex&gt; Moar.Tuple . from_list! ( [ { :a , 1 } , { :b , 2 } , { :a , 3 } ] ) ** (RuntimeError) Expected all items in the list to have have the same first element, but got: [:a, :b]","ref":"Moar.Tuple.html#from_list!/1","title":"Moar.Tuple.from_list!/1","type":"function"},{"doc":"Reduces a list of tuples to map where values are consolidated by the first element of each input tuple. iex&gt; Moar.Tuple . reduce ( [ { :ok , 1 } , { :ok , 2 } ] ) %{ ok : [ 1 , 2 ] } iex&gt; Moar.Tuple . reduce ( [ { :ok , 1 } , { :ok , 2 } , { :error , 3 } , { :ok , 4 } ] ) %{ ok : [ 1 , 2 , 4 ] , error : [ 3 ] }","ref":"Moar.Tuple.html#reduce/1","title":"Moar.Tuple.reduce/1","type":"function"},{"doc":"An assortment of useful functions. The docs can be found at https://hexdocs.pm/moar","ref":"readme.html","title":"Moar","type":"extras"},{"doc":"The package can be installed by adding moar to your list of dependencies in mix.exs : def deps do [ { :moar , &quot;~&gt; 1.25&quot; } ] end","ref":"readme.html#installation","title":"Moar - Installation","type":"extras"},{"doc":"Bunch Swiss","ref":"readme.html#similar-libraries","title":"Moar - Similar libraries","type":"extras"},{"doc":"Moar.Assertions assert_eq is a pipeable equality assertion, with options such as the ability to ignore order when comparing lists, ignore whitespace when comparing strings, returning a different value than what was passed in, and asserting that a value is within some delta (which can be a number or time duration). assert_recent asserts that a datetime is pretty close to now. assert_that asserts that a pre-condition and post-condition are true after performing an action. refute_that asserts that a condition didn't change after performing an action. Moar.Atom atomize when given an atom, returns it; when given a string, converts it to an atom, replacing non-alphanumeric characters with underscores. from_string and to_string convert between strings and atoms, and don't fail if you try to convert an atom to an atom or a string to a string. to_existing_atom is like String.to_existing_atom/1 but can accept an atom as a param. Moar.DateTime and Moar.NaiveDateTime add and subtract can add and subtract a Moar.Duration , which is a tuple with a time unit, like {27, :minute} . between? determines whether a datetime is between two other datetimes. from_iso8601! raises if the string is not in ISO 8601 format. recent? returns true if the given datetime was at most one minute ago. to_iso8601_rounded converts to an ISO 8601 string, truncated to the second. utc_now takes plus and minus options to get the current time plus or minus some duration. Moar.Difference a protocol that defines diff(a, b) along with implementations for datetimes. Moar.Duration is a {time, unit} tuple (like {27, :minute} ) and supports the regular System.time_unit/0 values and also :minute , :hour , and :day . ago returns the duration between a given datetime and now. approx shifts the duration to a simple approximate value. between returns the duration between two datetimes. convert converts a duration to a new time unit, returning only the value. format formats a duration in long ( &quot;3 seconds&quot; ) or short ( &quot;3s&quot; ) format with optional transformers and suffix. from_now returns a duration between now and a given datetime. humanize converts the duration to the highest possible time unit. shift , shift_up , and shift_down convert a duration to a new time unit. to_string renders a duration into a string like &quot;27 minutes&quot; . Moar.Enum at! is like Enum.at but raises if the index is out of bounds. compact rejects nil values. first! returns the first item or raises if there isn't a first item. is_map_or_keyword returns true if the value is a map or a keyword list (unfortunately cannot be used as a guard). isort and isort_by sort case-insensitively. take_at returns a list of elements at the given indices. tids extracts tid fields. ( tid is short for &quot;test id&quot; and the authors of Moar use tids extensively for testing.) Moar.File new_tempfile_path returns a new path for a tempfile, without creating it. write_tempfile writes some data to a new tempfile. Moar.Map atomize_key , atomize_keys , and deep_atomize_keys convert keys in a map from strings to atoms, and stringify_keys does the opposite. merge and deep_merge merge maps, converting enumerables into maps before merging. deep_merge also accepts a function to resolve value conflicts. put_if_blank puts a key/value pair into a map if the key is missing or its value is blank (via Moar.Term.blank?/1 ) rename_key and rename_keys rename keys in a map. transform transforms a key or multiple keys with a transformer function. Moar.Opts is meant to be used with function options. get extracts a value from opts, falling back to a default if the value is blank (via Moar.Term.blank?/1 ) take extracts multiple values from opts, falling back to defaults if the value is blank (via Moar.Term.blank?/1 ) Moar.Protocol implements! raises if a struct does not implement the given protocol. implements? returns true if a struct implements the given protocol. Moar.Random integer returns a random integer. string returns random base64- or base32-encoded string. Moar.Retry rescue_until and rescue_for run the given function repeatedly until it does not raise. retry_until and retry_for run the given function repeatedly until it returns a truthy value. Moar.String append_unless_blank appends a suffix to a string, unless the string is blank. inner_truncate removes the middle of a string to make it the desired length. secure_compare compares two strings in constant time. slug converts a string into a slug with a custom joiner character; dasherize and underscore are shortcuts for common slug formats. squish collapses consecutive whitespace characters. surround wraps a string with the given characters. to_case converts text to :camel_case , :lower_camel_case , or :snake_case . to_integer converts a string to an integer with a few handy options. trim is like String.trim/1 but handles nil values. truncate_at truncates a string at the last instance of a substring that results in the truncated string being shorter than a given length. Moar.Sugar error , noreply , and ok create tuples ( &quot;foo&quot; |&gt; error() -&gt; {:error, &quot;foo&quot;} ). error! and ok! unwrap tuples ( {:error, &quot;foo&quot;} |&gt; error!() -&gt; &quot;foo&quot; ). returning takes two arguments and returns the second one. Moar.Term blank? returns true if a term is nil , an empty string, a whitespace-only string, an empty list, or an empty map. present? is the opposite of blank? . presence returns a default if the argument is blank. Moar.Tuple from_list! converts a list of tuples like [{:ok, 1}, {:ok, 2}, {:ok, 3}] to a tuple like {:ok, [1, 2, 3]} reduce converts a list of tuples like [{:ok, 1}, {:error, 2}] to a map like %{ok: [1], error: [2]}","ref":"readme.html#a-quick-tour","title":"Moar - A quick tour","type":"extras"},{"doc":"Copyright 2022 synchronal.dev Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]